'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildTOC = exports.getLevel = exports.getClosestParent = exports.buildStructure = exports.defaultOpts = undefined;

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.default = toc;

var _pupa = require('pupa');

var _pupa2 = _interopRequireDefault(_pupa);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultOpts = {
  listTemplate: '<ul class="toc__list toc--level-{level}"></ul>',
  itemTemplate: '<li class="toc__item toc--level-{level}">{text}</li>',
  selector: '.toc__box'
};
exports.defaultOpts = defaultOpts;
function toc() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  opts = (0, _assign2.default)({}, defaultOpts, opts);
  return function ($, cheerio) {
    var structure = buildStructure($);
    var result = { html: buildTOC(opts, $, cheerio, structure) };
    return result;
  };
}

function buildStructure($) {
  var headlines = $('h1,h2,h3,h4,h5,h6');
  var latestMap = new _map2.default();
  var structure = [];

  headlines.each(function (idx, elem) {
    var level = getLevel(elem.name);
    var parent = getClosestParent(level, latestMap);
    var item = {
      level: level,
      parent: parent,
      text: $(elem).text(),
      children: []
    };

    if (parent) {
      parent.children.push(item);
    } else {
      structure.push(item);
    }
    latestMap.set(level, item);
  });

  return structure;
}

exports.buildStructure = buildStructure;


function getClosestParent(level, map) {
  var next = level - 1;
  while (!map.get(next) && next > 0) {
    next -= next;
  }

  if (next === 0) {
    return null;
  }
  return map.get(next);
}
exports.getClosestParent = getClosestParent;


function getLevel(name) {
  var matches = name.match(/\d+/);
  if (matches) {
    return Number(matches[0]);
  }
}
exports.getLevel = getLevel;


function buildTOC(opts, $, cheerio, structure) {
  var htmls = proc({
    children: structure,
    root: true
  });

  htmls.slice(1).reduce(function ($prev, next) {
    return $(next).insertAfter($prev);
  }, $(htmls[0]).prependTo($(opts.selector)));

  return $.html();

  function proc(_ref) {
    var children = _ref.children,
        _ref$$box = _ref.$box,
        $box = _ref$$box === undefined ? null : _ref$$box,
        _ref$root = _ref.root,
        root = _ref$root === undefined ? false : _ref$root;

    var htmls = children.map(function (item) {
      var $ = cheerio.load('', { decodeEntities: false });

      if (root) {
        var _list = (0, _pupa2.default)(opts.listTemplate, item);
        var _item = (0, _pupa2.default)(opts.itemTemplate, item);
        var $list = $(_list).appendTo($.root());
        $(_item).appendTo($list);
        if (item.children.length > 0) {
          $list.append(proc({
            children: item.children,
            $box: $list
          }));
        }
      } else {
        var _wrapper = (0, _pupa2.default)(opts.itemTemplate, { level: item.parent.level });
        var _rootList = (0, _pupa2.default)(opts.listTemplate, item);
        var _item2 = (0, _pupa2.default)(opts.itemTemplate, item);

        var $wrapper = $(_wrapper).appendTo($box);
        var _$list = $(_rootList).appendTo($wrapper);
        $(_item2).appendTo(_$list);
        if (item.children.length > 0) {
          _$list.append(proc({
            children: item.children,
            $box: _$list
          }));
        }
      }
      return $.html();
    });
    return htmls;
  }
}
exports.buildTOC = buildTOC;