'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

exports.default = toc;

var _pupa = require('pupa');

var _pupa2 = _interopRequireDefault(_pupa);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function toc($, cheerio) {
  var headlines = $('h1,h2,h3,h4,h5,h6');
  var structure = buildStructure($, headlines);
  var result = { html: buildTOC($, cheerio, structure) };
  return result;
}

function buildStructure($, headlines) {
  var latestMap = new _map2.default();
  var structure = [];

  headlines.each(function (idx, elem) {
    var level = getLevel(elem.name);
    var parent = getClosestParent(level, latestMap);
    var item = {
      level: level,
      parent: parent,
      text: $(elem).text(),
      children: []
    };

    if (parent) {
      parent.children.push(item);
    } else {
      structure.push(item);
    }
    latestMap.set(level, item);
  });

  return structure;
}

function getClosestParent(level, map) {
  var next = level - 1;
  while (!map.get(next) && next > 0) {
    next -= next;
  }

  if (next === 0) {
    return null;
  }
  return map.get(next);
}

function getLevel(name) {
  var matches = name.match(/\d+/);
  if (matches) {
    return Number(matches[0]);
  }
}

function buildTOC($, cheerio, structure) {
  var listTemplate = '<ul class="toc__list toc--level-{level}"></ul>';
  var itemTemplate = '<li class="toc__item toc--level-{level}">{text}</li>';

  var htmls = proc({
    children: structure,
    root: true
  });

  htmls.slice(1).reduce(function ($prev, next) {
    return $(next).insertAfter($prev);
  }, $(htmls[0]).prependTo($('.toc__box')));

  return $.html();

  function proc(_ref) {
    var children = _ref.children,
        _ref$$box = _ref.$box,
        $box = _ref$$box === undefined ? null : _ref$$box,
        _ref$root = _ref.root,
        root = _ref$root === undefined ? false : _ref$root;

    var htmls = children.map(function (item) {
      var $ = cheerio.load('', { decodeEntities: false });

      if (root) {
        var _list = (0, _pupa2.default)(listTemplate, item);
        var _item = (0, _pupa2.default)(itemTemplate, item);
        var $list = $(_list).appendTo($.root());
        $(_item).appendTo($list);
        if (item.children.length > 0) {
          $list.append(proc({
            children: item.children,
            $box: $list
          }));
        }
      } else {
        var _wrapper = (0, _pupa2.default)(itemTemplate, { level: item.parent.level });
        var _rootList = (0, _pupa2.default)(listTemplate, item);
        var _item2 = (0, _pupa2.default)(itemTemplate, item);

        var $wrapper = $(_wrapper).appendTo($box);
        var _$list = $(_rootList).appendTo($wrapper);
        $(_item2).appendTo(_$list);
        if (item.children.length > 0) {
          _$list.append(proc({
            children: item.children,
            $box: _$list
          }));
        }
      }
      return $.html();
    });
    return htmls;
  }
}